/**
 * @file song.h
 * @author Shinichiro Nakamura
 * @copyright
 * ===============================================================
 * A tiny MML parser (Version 0.4.0)
 * ===============================================================
 * Copyright (c) 2014-2015 Shinichiro Nakamura
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 */

#ifndef SONG_H
#define SONG_H

#include <avr/pgmspace.h>

/**
 * @brief Super Mario Bros. (Main Theme - Overworld)
 * @details
 * Original Music Composed by Koji Kondo
 * Transcribed/Arranged by Philip Kim
 * http://www.mariomayhem.com/downloads/sheet_music/super_mario_brothers_complete/smb1_complete.pdf
 */
const char song_over_world[] PROGMEM =
  "T180"

  /* This L0 is a trick for playing chords. */
  "L0"

  /* Intro */
  "O3D>F+>E8 O3D>F+>E8 R8 O3D>F+>E8 R8 O3D>F+>C8 O3D>F+>E4"
  "O3G>GB>G4 R4 O2G>>G4 R4"

  "["
    /* A */
    "O3G>E>C4 R8 O3E>CG4 R8 O3G>E4"
    "R8 O3F>CA4 O3G>DB4 O3G->D-B-8 O3F>CA4"
    "O3E>CG43 O3>CG>E43 O3>EG>G43 O3>F>CA4 O3>DA>F8 O3>EB>G8"
    "R8 O3>CA>E4 O3A>E>C8 O3B>F>D8 O3G>DB4 R8"
  "]"

  "["
    /* B */
    "O3C4 O3>>EG8 O3G>>D+F+8 O3>>DF8 O3>B>D+8 O3>C8 O3>>CE8"
    "O3F8 O3>EG+8 O3>FA8 O3>CG>C8 O3>C8 O3>CA8 O3F>E>C8 O3>F>D8"
    "O3C4 O3>>EG8 O3E>>E-G-8 O3>>DF8 O3>B>D+8 O3G8 O3>C>CE8"
    "R8 O3>>FG>C8 R8 O3>>FG>C8 O3>>FG>C8 R8 O3G4"
    "O3C4 O3>>EG8 O3G>>D+F+8 O3>>DF8 O3>B>D+8 O3>C8 O3>>CE8"
    "O3F8 O3>EG+8 O3>FA8 O3>CG>C8 O3>C8 O3>CA8 O3F>E>C8 O3>F>D8"
    "O3C4 O3A->A->E-8 R8 R8 O3B->F>D8 R8 R8"
    "O3>CE>C8 R8 R8 O3G8 O3G4 O3C8 R8"
  "]"

  /* C */
  "O2A->>A->C8 O3>A->C8 R8 O3E->A->C8 R8 O3>A->C8 O3A->B->D4"
  "O3G>G>E8 O3>E>C8 R8 O3C>EA8 O3>CG4 O2G4"
  "O2A->>A->C8 O3>A->C8 R8 O3E->A->C8 R8 O3>A->C8 O3A->B->D8 O3>G>E8"
  "O3G4 R8 O3C8 R4 O2G4"
  "O2A->>A->C8 O3>A->C8 R8 O3E->A->C8 R8 O3>A->C8 O3A->B->D4"
  "O3G>G>E8 O3>E>C8 R8 O3C>EA8 O3>CG4 O2G4"
  "O3D>F+>E8 O3D>F+>E8  R8 O3D>F+>E8 R8 O3D>F+>C8 O3D>F+>E4"
  "O3G>GB>G4 R4 O2G>G4 R4"

  "["
    /* A */
    "O3G>E>C4 R8 O3E>CG4 R8 O3G>E4"
    "R8 O3F>CA4 O3G>DB4 O3G->D-B-8 O3F>CA4"
    "O3E>CG43 O3>CG>E43 O3>EG>G43 O3>F>CA4 O3>DA>F8 O3>EB>G8"
    "R8 O3>CA>E4 O3A>E>C8 O3B>F>D8 O3G>DB4 R8"
  "]"

  "["
    /* D */
    "O3C>>CE8 O3>A>C4 O3F+>EG8 O3G4 O3>CEG+4"
    "O3F>FA8 O3>>CF8 O3F8 O3>>CF8 O3>CFA8 O3>C8 O3F4"
    "O3D>GB43 O3>>FA43 O3F>>FA43 O3G>>FA43 O3>>EG43 O3B>>DF43"
    "O3G>>CE8 O3>A>C8 O3G8 O3>FA8 O3>CEG8 O3>C8 O3G4"
    "O3C>>CE8 O3>A>C4 O3F+>EG8 O3G4 O3>CEG+4"
    "O3F>FA8 O3>>CF8 O3F8 O3>>CF8 O3>CFA8 O3>C8 O3F4"
    "O3G>GB8 O3G>>DF8 R8 O3G>>DF8 O3G>>DF43 O3A>>CE43 O3B>B>D43"
    "O3>CG>C8 O3>CE>C8 O3G>E>C8 O3G>E>C8 O3C>C4 R4"
  "]"

  /* C' */
  "O2A->>A->C8 O3>A->C8 R8 O3E->A->C8 R8 O3>A->C8 O3A->B->D4"
  "O3G>G>E8 O3>E>C8 R8 O3C>EA8 O3>CG4 O2G4"
  "O2A->>A->C8 O3>A->C8 R8 O3E->A->C8 R8 O3>A->C8 O3A->B->D8 O3>G>E8"
  "O3G4 O3>>E8 O3C>>G8 O3>>>E8 O3>>>C8 O2G>>>D8 O3>>>G8"
  "O2A->>A->C8 O3>A->C8 R8 O3E->A->C8 R8 O3>A->C8 O3A->B->D4"
  "O3G>G>E8 O3>E>C8 R8 O3C>EA8 O3>CG4 O2G4"
  "O3D>F+>E8 O3D>F+>E8  R8 O3D>F+>E8 R8 O3D>F+>C8 O3D>F+>E4"
  "O3G>GB>G4 R4 O2G>G4 R4"

  "["
    /* D */
    "O3C>>CE8 O3>A>C4 O3F+>EG8 O3G4 O3>CEG+4"
    "O3F>FA8 O3>>CF8 O3F8 O3>>CF8 O3>CFA8 O3>C8 O3F4"
    "O3D>GB43 O3>>FA43 O3F>>FA43 O3G>>FA43 O3>>EG43 O3B>>DF43"
    "O3G>>CE8 O3>A>C8 O3G8 O3>FA8 O3>CEG8 O3>C8 O3G4"
    "O3C>>CE8 O3>A>C4 O3F+>EG8 O3G4 O3>CEG+4"
    "O3F>FA8 O3>>CF8 O3F8 O3>>CF8 O3>CFA8 O3>C8 O3F4"
    "O3G>GB8 O3G>>DF8 R8 O3G>>DF8 O3G>>DF43 O3A>>CE43 O3B>B>D43"
    "O3>CG>C8 O3>CE>C8 O3G>E>C8 O3G>E>C8 O3C>C4 R4"
  "]"

  /* E */
  "O3G>E>C4 R8 O3E>CG4 R8 O3CG>E4"
  "T170 O2F>C>FA43 O2F>C>FB43 O2F>C>FA43 T160 O2D-A->>FA-43 O2D-A->>FB-43 O2D-A->>FA-43"
  "T150 O2CG>>EG8 O2CG>>DG8 O2CG>>EG2."

  "R1"

  ;

/**
 * @brief Super Mario Bros. (Level Clear)
 * @details
 * Original Music Composed by Koji Kondo
 * Transcribed/Arranged by Philip Kim
 * http://www.mariomayhem.com/downloads/sheet_music/super_mario_brothers_complete/smb1_complete.pdf
 */
const char song_level_clear[] PROGMEM =
  "T140"

  /* This L0 is a trick for playing chords. */
  "L0"

  "O3CG83 O3E>C83 O3G>E83 O3>CG83 O3G>E>C83 O3>CG>E83 O3>E>CG4 O3>CG>E4"
  "O3CF-83 O3E->C83 O3A->E-83 O3>CA-83 O3A>E->C83 O3>CA->E-83 O3>E->CA-4 O3>CA->E-4"
  "O3DB-83 O3F>D83 O3B->F83 O3>DB-83 O3B>F>D83 O3>DB->F83 O3>F>DB-4 O3D>D>DFB-83 O3D>D>DFB-83 O3D>D>DFB-83"
  "O3C>C>CE>C1"

  "R1"

  ;

#endif

