/**
 * @file note.cpp
 * @author Shinichiro Nakamura
 * @copyright
 * ===============================================================
 * A tiny MML parser (Version 0.4.0)
 * ===============================================================
 * Copyright (c) 2014-2015 Shinichiro Nakamura
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 */

#include <stdint.h>
#include <Arduino.h>
#include "note.h"

#define SOUND_SYSTEM_TONE(FREQ, MS) do { tone(11, (FREQ)); delay(MS); } while (0)
#define SOUND_SYSTEM_REST(MS)       do { noTone(11); delay(MS); } while (0)

const uint16_t notefreq[] PROGMEM = {
     32,    34,    36,    38,    41,    43,    46,    48,    51,    55,    58,    61,
     65,    69,    73,    77,    82,    87,    92,    97,   103,   110,   116,   123,
    130,   138,   146,   155,   164,   174,   184,   195,   207,   220,   233,   246,
    261,   277,   293,   311,   329,   349,   369,   391,   415,   440,   466,   493,
    523,   554,   587,   622,   659,   698,   739,   783,   830,   880,   932,   987,
   1046,  1108,  1174,  1244,  1318,  1396,  1479,  1567,  1661,  1760,  1864,  1975,
   2093,  2217,  2349,  2489,  2637,  2793,  2959,  3135,  3322,  3520,  3729,  3951,
   4186,  4434,  4698,  4978,  5274,  5587,  5919,  6271,  6644,  7040,  7458,  7902,
   8372,  8869,  9397,  9956, 10548, 11175, 11839, 12543, 13289, 14080, 14917, 15804,
  16744, 17739, 18794, 19912, 21096, 22350, 23679, 25087, 26579, 28160, 29834, 31608,
  33488, 35479, 37589, 39824, 42192, 44701, 47359, 50175,
};

static uint32_t get_note_length_ms(NOTE *p, uint32_t note_ticks)
{
  return (60000) * note_ticks / p->bpm / p->bticks;
}

void note_init(NOTE *p, int bpm, int bticks)
{
  p->bpm = bpm;
  p->bticks = bticks;
}

void note_tone(NOTE *p, int number, int ticks)
{
  uint16_t freq = pgm_read_word(&(notefreq[number]));
  uint32_t ms = get_note_length_ms(p, ticks);
  SOUND_SYSTEM_TONE(freq, ms);
}

void note_rest(NOTE *p, int ticks)
{
  uint32_t ms = get_note_length_ms(p, ticks);
  SOUND_SYSTEM_REST(ms);
}

