/**
 * @file main.c
 * @author Shinichiro Nakamura
 * @copyright
 * ===============================================================
 * A tiny MML parser (Version 0.4.0)
 * ===============================================================
 * Copyright (c) 2014-2015 Shinichiro Nakamura
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 */

#include <stdio.h>
#include "mml.h"

static void mml_callback(MML_INFO *p, void *extobj)
{
  static int cnt = 0;

  if (cnt == 0) {
    /*
     * I know the external object is a pointer to a text string in this sample program.
     */
    printf("============================================\n");
    printf(" %s", (char *)extobj);
    printf("============================================\n");
  }

  printf("%3d : ", ++cnt);
  switch (p->type) {
    case MML_TYPE_NOTE:
      {
        MML_ARGS_NOTE *args = &(p->args.note);
        printf("[NOTE  : Number=%3d, Ticks=%4d]\n", args->number, args->ticks);
      }
      break;
    case MML_TYPE_REST:
      {
        MML_ARGS_REST *args = &(p->args.rest);
        printf("[REST  :             Ticks=%4d]\n", args->ticks);
      }
      break;
    case MML_TYPE_TEMPO:
      {
        MML_ARGS_TEMPO *args = &(p->args.tempo);
        printf("[TEMPO : Value=%3d]\n", args->value);
      }
      break;
    case MML_TYPE_LENGTH:
      {
        MML_ARGS_LENGTH *args = &(p->args.length);
        printf("[LENGTH: Value=%3d]\n", args->value);
      }
      break;
    case MML_TYPE_VOLUME:
      {
        MML_ARGS_VOLUME *args = &(p->args.volume);
        printf("[VOLUME: Value=%3d]\n", args->value);
      }
      break;
    case MML_TYPE_OCTAVE:
      {
        MML_ARGS_OCTAVE *args = &(p->args.octave);
        printf("[OCTAVE: Value=%3d]\n", args->value);
      }
      break;
    case MML_TYPE_OCTAVE_UP:
      {
        MML_ARGS_OCTAVE *args = &(p->args.octave_up);
        printf("[OCTAVE: Value=%3d(Up)]\n", args->value);
      }
      break;
    case MML_TYPE_OCTAVE_DOWN:
      {
        MML_ARGS_OCTAVE *args = &(p->args.octave_down);
        printf("[OCTAVE: Value=%3d(Down)]\n", args->value);
      }
      break;
  }
}

int main(int argc, char **argv)
{
  MML mml;
  MML_OPTION mml_opt;
  MML_RESULT mr;
  void *mml_extobj;

  if (argc != 2) {
    printf("mmltest [MML]\n");
    return 0;
  }

  mml_extobj = (void *)"Sample Program for 'A tiny MML parser'\n Copyright(C) 2014-2015 Shinichiro Nakamura\n";
  mr = mml_init(&mml, mml_callback, mml_extobj);
  if (mr != MML_RESULT_OK) {
    printf("[ERROR] : init failed.\n");
  }

  MML_OPTION_INITIALIZER_DEFAULT(&mml_opt);
  mr = mml_setup(&mml, &mml_opt, argv[1]);
  if (mr != MML_RESULT_OK) {
    printf("[ERROR] : setup failed (input text='%s').\n", argv[1]);
  }

  while ((mr = mml_fetch(&mml)) == MML_RESULT_OK) {
  }

  printf("[%s] : Final code is %d.\n", (mr == MML_RESULT_EOT) ? "OK" : "ERROR", (int)mr);

  return 0;
}

